const pm2 = require("pm2");
const express = require("express");
const path = require("path");
const os = require("os");
const fs = require("fs")
const cors = require("cors");
const config = require("./config.json")

const app = express();

app.use(cors());


const shutdown = () => {
    pm2.disconnect(() => {
        console.log("[PM2] >> Disconnected.");
        process.exit(0);
      });
}

// Handle app termination
process.on('SIGINT', shutdown);
process.on('SIGTERM', shutdown);

pm2.connect(function(err) {
    if (err) {
        console.log("[PM2] >> Error connecting to pm2:", err);
        process.exit(2);
    }
    console.log("[PM2] >> Connected to pm2");
})

app.use((req, res, next) => {
    console.log(`[${new Date().toUTCString()}] >> ${req.method} : ${req.url} @ ${req.socket.remoteAddress}`)
    if (config.TOKEN) {
        if (req.headers.access_token !== config.TOKEN) {
            console.log("Access Denied.")
            return res.status(401).send({success: false, message: "Unauthorized"});
        }
        next();
    } else {
        next()
    }
  })

// API Endpoint to return PM2 process list
app.get('/processes', (req, res) => {
    pm2.list((err, processDescriptionList) => {
      if (err) {
        return res.status(500).json({ success: false, error: err.message });
      }
      res.status(200).json({success: true, processes: processDescriptionList});
    });
});

app.post('/:id/stop', (req, res) => {
    pm2.stop(req.params.id, (err, proc) => {
        if (err) {
            return res.status(500).json({ success: false, error: err.message });
        }
        return res.status(200).json({ success: true, processes: proc });
    })
})

app.get('/:id/info', (req, res) => {
    pm2.describe(req.params.id, (err, proc) => {
        if (err) {
            return res.status(500).json({ success: false, error: err.message });
        }
        return res.status(200).json({ success: true, processes: proc });
    })
})

app.post('/:id/restart', (req, res) => {
    pm2.restart(req.params.id, (err, proc) => {
        if (err) {
            return res.status(500).json({ success: false, error: err.message });
        }
        return res.status(200).json({ success: true, processes: proc });
    })
})

app.post('/:id/resume', (req, res) => {
    pm2.start(req.params.id, (err, proc) => {
        if (err) {
            return res.status(500).json({ success: false, error: err.message });
        }
        return res.status(200).json({ success: true, processes: proc });
    })
})

app.post('/:id/logs', (req, res) => {
    const id = req.params.id;
    const homeDir = os.homedir();

    const successReturn = {
        success: true,
        logs: [],
        error: []
    }

    let filePath = path.join(homeDir, ".pm2", "logs", `${id}-out.log`);
    fs.readFile(filePath, 'utf8', (err, data) => {
        if (err) {
            // Handle errors if file reading fails
            res.status(500).json({ success: false, error: `File Error: ${err}` });
            return;
        }

        // Split the file content by newlines
        const lines = data.split('\n');

        // Get the last 50 lines
        // const last50Lines = lines.slice(-50).join('\n');
        const last50Lines = lines.slice(-50);

        // Send the last 50 lines as the response
        successReturn.logs = last50Lines;
    
        filePath = path.join(homeDir, ".pm2", "logs", `${id}-error.log`);
        fs.readFile(filePath, 'utf8', (err, data) => {
            if (err) {
                // Handle errors if file reading fails
                res.status(500).json({ success: false, error: `File Error: ${err}` });
                return;
            }
    
            // Split the file content by newlines
            const lines = data.split('\n');
    
            // Get the last 50 lines
            // const last50Lines = lines.slice(-50).join('\n');
            const last50Lines = lines.slice(-50);
    
            successReturn.error = last50Lines        
            res.status(200).json(successReturn);
        });
    });

})

app.post('/:id/error', (req, res) => {
    const id = req.params.id;
    const homeDir = os.homedir();

    const filePath = path.join(homeDir, ".pm2", "logs", `${id}-error.log`);
    fs.readFile(filePath, 'utf8', (err, data) => {
        if (err) {
            // Handle errors if file reading fails
            res.status(500).json({ success: false, error: `File Error: ${err}` });
            return;
        }

        // Split the file content by newlines
        const lines = data.split('\n');

        // Get the last 50 lines
        // const last50Lines = lines.slice(-50).join('\n');
        const last50Lines = lines.slice(-50);

        // Send the last 50 lines as the response
        res.status(200).json({ success: true, logs: last50Lines });
    });
})

// Start server
const PORT = config.PORT || 2222;
app.listen(PORT, () => {
  console.log(`[Express] >> Server running on port ${PORT}`);
});